<?php

/**
 * @file
 * proc.functions.inc
 */

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Component\Utility\Crypt;
use Drupal\Component\Utility\Html;
use Drupal\Core\Site\Settings;
use Drupal\user\Entity\User;

/**
 * Helper function for defining labels used in JS.
 *
 * @return array
 *   Array containing labels.
 */
function _proc_js_labels(): array {
  return [
    'proc_fileapi_err_msg' => t('Your browser does not support file API.'),
    'proc_button_state_processing' => t('Processing...'),
    'proc_max_encryption_size' => t('Error. Maximum file size allowed:'),
    'proc_max_encryption_size_unit' => t('bytes'),
    'proc_save_button_label' => t('Save'),
    'proc_size' => t('Size:'),
    'proc_type' => t('Type:'),
    'proc_last_modified' => t('Last modified:'),
    // Visible as data-drupal-selector=password-strength-text.
    'proc_minimal_password_strength' => t('Strong'),
    'proc_generate_keys_submit_label' => t('Generate encryption keys'),
    'proc_submit_saving_state' => t('Saving...'),
    'proc_password_match' => t('You must type in both password fields the same password'),
    'proc_password_required' => t('Password is required.'),
    'proc_introducing_decryption' => t('Introducing key passphrase for decryption...'),
    'proc_open_file_state' => t('Open'),
    'proc_decryption_success' => t('Decryption successful.'),
    'proc_decryption_size_mismatch' => t('Error: size mismatch.'),
    'proc_button_update_label' => t('Update'),
    'proc_introducing_signature' => t('Introducing key passphrase for signature...'),
    'proc_pass_weak' => t('Your password must be stronger.'),
    'proc_caches_unsupported' => t('Error: this browser does not support cache API. Please use an updated browser.'),
  ];
}

/**
 * Helper common to get proc decryption form data.
 *
 * @param string|null $privkey
 *   Current user encrypted privkey.
 * @param bool $keyGeneration
 *   Flag for key generation.
 *
 * @return array
 *   Array containing proc form data.
 *
 * @SuppressWarnings(PHPMD.BooleanArgumentFlag)
 * @SuppressWarnings(PHPMD.StaticAccess)
 */
function _proc_common_get_decryption_form_data(string $privkey = NULL, bool $keyGeneration = FALSE): array {
  // Get the timestamp of the key creation:
  $user = User::load(Drupal::currentUser()->id());
  $user_uid = $user->id();
  $hash = Settings::getHashSalt();
  $suffix = $user_uid;
  // Load the keyring of the current user:
  try {
    $keyring = _proc_get_keys($user_uid, 'user_id');
  }
  catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
    // Log error:
    \Drupal::logger('proc')->error($e->getMessage());
    return [];
  }

  $createdTimestamp = $keyring['created'] ?? \Drupal::time()->getRequestTime();

  // Add a hook alter invocation for allowing the change of the suffix based on
  // the context, iff this is not being called in the context of key generation:
  if (!$keyGeneration) {
    \Drupal::moduleHandler()->alter('proc_hash_salt_legacy_extension', $suffix, $createdTimestamp);
  }

  $hash_base64 = Crypt::hashBase64($hash . $suffix);

  return [
    'proc_uid' => $user_uid,
    'proc_pass' => $hash_base64,
    'proc_email' => $user->getEmail(),
    'proc_name' => $user->getAccountName(),
    'proc_privkey' => $privkey,
  ];
}

/**
 * Helper function to get unique CSV arguments.
 *
 * @param string $csv_data
 *   CSV data sent as URL argument.
 *
 * @return array|null
 *   An array containing the unique entries of the CSV, or NULL if input is not
 *   set.
 *
 * @SuppressWarnings(PHPMD.StaticAccess)
 */
function _proc_get_csv_argument(string $csv_data): ?array {
  if (!isset($csv_data)) {
    return NULL;
  }

  // Remove query string from the path if any.
  $csv_data = preg_replace('/\?.+/', '', $csv_data);
  $csv_data = Html::escape($csv_data);
  $csv_values = str_getcsv($csv_data);
  $csv_data = [];

  foreach ($csv_values as $csv_value) {
    if (is_numeric($csv_value)) {
      $csv_value = (int) $csv_value;
      $csv_data[] = $csv_value;
    }
  }

  if (!empty($csv_data)) {
    return array_values(array_unique($csv_data));
  }

  return NULL;
}

/**
 * Helper common function for getting ciphers for decryption, signature.
 *
 * @param array $cids
 *   List of cids.
 * @param bool|null $metadata_only
 *   Whether to return only metadata.
 *
 * @return array
 *   Array containing ciphers, encrypted privkey and recipient's keyring type.
 *
 * @SuppressWarnings(PHPMD.StaticAccess)
 */
function _proc_common_get_ciphers(array $cids, $metadata_only = NULL): array {

  $user = Drupal::currentUser();
  $user_id = $user->id();

  try {
    $keyring = _proc_get_keys($user_id, 'user_id');
  }
  catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
    // Log error:
    \Drupal::logger('proc')->error($e->getMessage());
    return [];
  }

  $privkey = $keyring['encrypted_private_key'];
  $pubkey = $keyring['public_key'];

  $cid_cipher = [];

  foreach ($cids as $cid) {
    $cid_cipher[$cid] = _proc_get_cipher((int) $cid, NULL, NULL, $metadata_only);
  }

  return [
    'privkey' => $privkey,
    'keyring_cid' => $keyring['keyring_cid'],
    'ciphers' => $cid_cipher,
    'keyring_type' => $keyring['keyring_type'],
    'pubkey' => $pubkey,
  ];
}

/**
 * Helper function for getting keyring data.
 *
 * @param string $item_id
 *   User ID | Proc ID.
 * @param string|null $type
 *   user_id | id | NULL.
 *
 * @return array
 *   Array containing pubkey and encrypted privkey.
 *
 * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
 * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
 */
function _proc_get_keys(string $item_id, string $type = NULL): array {
  if (!isset($type)) {
    $type = 'user_id';
  }

  try {
    $query = Drupal::entityQuery('proc')
      ->accessCheck(TRUE)
      ->condition('type', 'cipher', '!=')
      ->condition('status', 1)
      ->condition($type, $item_id)
      ->sort('id', 'DESC')
      ->range(0, 1);
  }
  catch (\Exception $e) {
    // Log error:
    \Drupal::logger('proc')->error($e->getMessage());
    return [];
  }

  $key_id = $query->execute();
  if (empty($key_id)) {
    return [];
  }
  $key_id = array_values($key_id)[0];
  $entity = Drupal::entityTypeManager()->getStorage('proc')->load($key_id);
  // Private key:
  $keyring_keys = [];
  $keyring_keys['encrypted_private_key'] = $entity->get('armored')
    ->getValue()[0]['privkey'];
  $keyring_keys['public_key'] = $entity->get('armored')
    ->getValue()[0]['pubkey'];
  $keyring_keys['created'] = $entity->get('created')->getValue()[0]['value'];
  $keyring_keys['changed'] = $entity->get('changed')->getValue()[0]['value'];
  $keyring_keys['keyring_cid'] = $key_id;
  $keyring_keys['keyring_type'] = $entity->get('type')->getValue()[0]['value'];

  return $keyring_keys;
}

/**
 * Helper function to get given ciphertext decryptable by the current user.
 *
 * @param string $cid
 *   Ciphertext ID (proc entity).
 * @param mixed|null $form
 *   Optional form parameter.
 * @param mixed|null $form_state
 *   Optional form state parameter.
 * @param bool|null $metadata_only
 *   Whether to return only metadata.
 *
 * @return array
 *   Array containing ciphertext data.
 */
function _proc_get_cipher(string $cid, mixed $form = NULL, mixed $form_state = NULL, bool $metadata_only = NULL): array {
  try {
    $entity = Drupal::entityTypeManager()->getStorage('proc')->load($cid);
  }
  catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
    // Log error:
    \Drupal::logger('proc')->error($e->getMessage());
    return [];
  }
  $recipient_id = _get_recipient_ids($entity);
  $user_id = Drupal::currentUser()->id();

  $cipher_text_data = [];
  if (!$metadata_only && isset($entity->get('armored')->getValue()[0]['cipher_fid'])) {
    $cipher_text_data['cipher_text'] = $entity->get('armored')->getValue()[0]['cipher_fid'];
  }

  if (in_array($user_id, $recipient_id) || $metadata_only) {
    $cipher_text_data = array_merge($cipher_text_data, _get_metadata($entity, $cid, $recipient_id));
  }

  return $cipher_text_data;
}

/**
 * Get recipient IDs from the entity.
 *
 * @param object $entity
 *   The proc entity.
 *
 * @return array
 *   Array of recipient IDs.
 */
function _get_recipient_ids(object $entity): array {
  $recipients = $entity->get('field_recipients_set')->getValue();
  $recipient_id = [];
  foreach ($recipients as $recipient) {
    $recipient_id[] = $recipient['target_id'];
  }
  return $recipient_id;
}

/**
 * Get metadata from the entity.
 *
 * @param object $entity
 *   The proc entity.
 * @param string $cid
 *   Ciphertext ID (proc entity).
 * @param array $recipient_id
 *   Array of recipient IDs.
 *
 * @return array
 *   Array containing metadata.
 *
 * @SuppressWarnings(PHPMD.StaticAccess)
 */
function _get_metadata(object $entity, string $cid, array $recipient_id): array {
  $meta = $entity->get('meta')->getValue()[0];
  return [
    'source_file_name' => isset($meta['source_file_name']) ? Html::escape($meta['source_file_name']) : '',
    'source_file_size' => isset($meta['source_file_size']) ? Html::escape($meta['source_file_size']) : '',
    'cipher_cid' => $cid,
    'proc_owner_uid' => $entity->get('user_id')->getValue()[0]['target_id'],
    'proc_recipients' => $recipient_id,
    'changed' => $entity->get('changed')->getValue()[0]['value'],
  ];
}

/**
 * Helper function to get max file size for encryption in bytes.
 *
 * @return float|int
 *   Dynamically defined max file size.
 */
function _proc_get_post_max_size_bytes(): float|int {
  $post_max_size = substr_replace(ini_get('post_max_size'), "", -1);
  return $post_max_size * 1000000;
}
