/**
 * @file
 * Decrypts cipher texts in a list.
 */
import { feedPassCache } from './proc-feed-pass-cache-module.js';

(function($, Drupal, once) {
  Drupal.behaviors.ProcBehavior = {
    async attach(context, settings) {
      if (context.id !== '') {
        const procItems = document.querySelectorAll('[data-proc-list-item^="proc-list-item"]');
        const procFormat = 'binary';
        const cipherTexts = [];
        const cipherTextTypes = [];
        procItems.forEach((procItem) => {
          const cipherText = procItem.getAttribute(
            'data-proc-list-item-cipher-text',
          );
          const cipherTextType = procItem.getAttribute(
            'data-proc-list-item-type',
          );
          cipherTexts.push(cipherText);
          cipherTextTypes.push(cipherTextType);

        });
        const _navigator = {};
        for (const i in navigator) _navigator[i] = navigator[i];
        const passwordSessionCachingSalt = JSON.stringify(_navigator);
        const cachedPassword = sessionStorage.getItem(
          `proc.password_cache.key_user_id.${drupalSettings.user.uid}`,
        );
        if (!cachedPassword) {
          // There is no cached password.
          const dialogDiv = $(document.createElement('div'));
          dialogDiv.attr('id', 'proc-decrypt-dialog');
          const passwordField = $(document.createElement('input'));
          passwordField.attr('type', 'password');
          passwordField.attr('id', 'password');
          const passwordLabel = $(document.createElement('label'));
          passwordLabel.attr('for', 'password');
          passwordLabel.html('Password');
          passwordField[0].addEventListener('input', function (evt) {
            let pass = this.value;
            let result = decryptListItems(pass);
            result.then(function (result) {
              if (result) {
                feedPassCache('1', drupalSettings.proc.proc_keyring_type, pass, passwordSessionCachingSalt, openpgp);
              }
            });
          });
          dialogDiv.append(passwordLabel);
          dialogDiv.append(passwordField);
          dialogDiv.dialog();
        }

        /**
         * Decrypts the private key.
         *
         * Decrypts the private key given a correct passphrase of symmetric encryption.
         *
         * @param password
         *   Password that was typed in or taken from cached.
         */
        async function decryptPrivateKey(password) {
          const passphrase = drupalSettings.proc.proc_pass.concat(password);
          const privateKey = await openpgp.readPrivateKey({
            armoredKey: drupalSettings.proc.proc_privkey,
          });
          return await openpgp
            .decryptKey({
              privateKey,
              passphrase,
            })
            .catch(function (err) {
              const decryptionLinkElement = $('a#decryption-link');
              $('form#-proc-decrypt-to-file').prepend(
                `<div class="messages error">${Drupal.t(err)}</div>`,
              );
            });
        }

        /**
         * Decrypts all listed items.
         *
         * Decrypts all listed items with the decrypted private key.
         *
         * @param decryptedPrivateKey
         *   Symmetrically decrypted private key.
         */
        async function decryptAllItems(decryptedPrivateKey) {
          for (const cipherText of cipherTexts) {
            const cipherTextIndex = cipherTexts.indexOf(cipherText);

            if (cipherText) {
              const message = await openpgp.readMessage({
                armoredMessage: cipherText,
              });
              const decrypted = await openpgp
                .decrypt({
                  decryptionKeys: decryptedPrivateKey,
                  message,
                  format: procFormat,
                })
                .catch(function (err) {
                  $('.messages').after(
                    `<div class="messages messages--error proc--error" id="proc-decrypting-status">${Drupal.t(
                      err,
                    )}</div>`,
                  );
                });
              if (decrypted) {
                await renderMedia(cipherTextIndex, decrypted);
              }
            }
          }
        }

        /**
         * Decrypts all listed items from uncached password dialog.
         *
         * Decrypts all listed items with a correct passphrase of symmetric encryption.
         *
         * @param password
         *   Passphrase for symmetrical decryption of private key.
         */
        async function decryptListItems(password) {
          const decryptedPrivateKey = await decryptPrivateKey(password);
          if (decryptedPrivateKey) {
            await decryptAllItems(decryptedPrivateKey);
            // Close dialog.
            $('#proc-decrypt-dialog').dialog('close');
            return true;
          }
        }

        if (cachedPassword) {
          try {
            await (async () => {
              const messageSource = await openpgp.readMessage({
                armoredMessage: sessionStorage.getItem(
                  `proc.password_cache.key_user_id.${drupalSettings.user.uid}`,
                ),
              });
              const { data: secretPassString } = await openpgp.decrypt({
                message: messageSource,
                passwords: [passwordSessionCachingSalt],
              });
              const decryptedPrivateKey = await decryptPrivateKey(
                secretPassString,
              );

              await decryptAllItems(decryptedPrivateKey);
            })();
          } catch (err) {
            // This error may happen because of a change in the browser fingerprint.
            console.info(err);
          }
        }

        /**
         * Renders the media according to mime type.
         *
         * @param cipherTextIndex
         * @param decrypted
         * @returns {Promise<void>}
         */
        async function renderMedia(cipherTextIndex, decrypted) {
          let procElement = document.querySelectorAll('[data-proc-list-item^="proc-list-item"]')[cipherTextIndex];
          let contentType = cipherTextTypes[cipherTextIndex];
          let data = new Blob([decrypted.data]);
          let url = URL.createObjectURL(data);
          const setTextContent = (element, text) => {
            if (element.innerText) {
              element.innerText = text;
            } else {
              element.value = text;
            }
          };
          const renderMedia = (tagName) => {
            const mediaElement = document.createElement(tagName.toUpperCase());
            mediaElement.src = url;
            mediaElement.controls = true;
            if (!procElement.innerHTML.includes('blob')) {
              const wrapper = document.createElement('div');
              wrapper.appendChild(mediaElement);
              procElement.appendChild(wrapper);
            }
          };
          if (contentType === 'text/plain') {
            const string = new TextDecoder().decode(decrypted.data);
            setTextContent(procElement, string);
          } else if (contentType.includes('image')) {
            const image = new Image();
            image.src = url;
            if (!procElement.innerHTML.includes('blob')) {
              const wrapper = document.createElement('div');
              wrapper.appendChild(image);
              procElement.appendChild(wrapper);
            }
          } else if (contentType.includes('video')) {
            renderMedia("video");
          } else if (contentType.includes('audio')) {
            renderMedia("audio");
          } else {
            await linkUnrecognizedMimeTypes(decrypted, procElement);
          }
        }

        /**
         * Links unrecognized mime types.
         *
         * @param decrypted
         * @param procElement
         * @returns {Promise<void>}
         */
        async function linkUnrecognizedMimeTypes(decrypted, procElement) {
          // Add temporary download link the same way as in proc-decrypt.js.
          const temporaryDownloadLink = document.createElement('a');
          const plaintext = decrypted.data;
          const blob = new Blob([plaintext], {
            type: 'application/octet-binary',
            endings: 'native',
          });
          temporaryDownloadLink.setAttribute('href', URL.createObjectURL(blob));
          temporaryDownloadLink.setAttribute(
            'download',
            procElement.innerText,
          );
          temporaryDownloadLink.innerText = Drupal.t('Download');
          // If it has not yet been added.
          if (!procElement.innerHTML.includes('blob')) {
            const wrapper = document.createElement('div');
            wrapper.appendChild(temporaryDownloadLink);
            procElement.appendChild(wrapper);
          }
        }
      }
    },
  };
})(jQuery, Drupal, once);
